package com.doc.biz.controller;

import com.doc.biz.domain.DocDir;
import com.doc.biz.domain.DocInfo;
import com.doc.biz.domain.DocSpace;
import com.doc.biz.service.*;
import com.doc.biz.vo.DocInfoVO;
import com.doc.biz.vo.DocumentVO;
import com.doc.common.annotation.Log;
import com.doc.common.config.EsConfig;
import com.doc.common.core.controller.BaseController;
import com.doc.common.core.domain.AjaxResult;
import com.doc.common.core.page.TableDataInfo;
import com.doc.common.enums.BusinessType;
import com.doc.common.utils.SecurityUtils;
import com.doc.common.utils.bean.BeanUtils;
import com.doc.common.utils.file.FileUtils;
import com.doc.common.utils.poi.ExcelUtil;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.elasticsearch.NoSuchIndexException;
import org.springframework.util.ResourceUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.FileInputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 文件基本信息表Controller
 *
 * @author wukai
 * @date 2023-08-15
 */
@Api(tags = "文件基本信息表")
@RestController
@RequestMapping("/biz/info")
@Slf4j
public class DocInfoController extends BaseController {
    @Resource
    private IDocInfoService docInfoService;
    @Resource
    private IMongoService mongoService;
    @Resource
    private IDocDirService dirService;
    @Resource
    private EsConfig esConfig;
    @Resource
    private IEsDocInfoService esDocInfoService;
    @Resource
    private IDocFavoriteService favoriteService;

    /**
     * 文件上传
     *
     * @param file
     * @return
     */
    @ApiOperation("文件上传-单文件")
    @PostMapping("/upload")
    public AjaxResult uploadFile(@ApiParam(value = "文件", required = true) @RequestPart(value = "file") MultipartFile file, @ApiParam(value = "空间ID", required = true) @RequestParam Long spaceId, @ApiParam(value = "目录ID", required = true) @RequestParam Long dirId) {
        try {
            DocumentVO vo = mongoService.uploadFile(file);
            process(vo, spaceId, dirId);
            return success();
        } catch (Exception e) {
            log.error("文件上传失败：", e);
            return error(e.getMessage());
        }
    }

    /**
     * 多文件上传
     *
     * @param files
     * @return
     */
    @ApiOperation("文件上传-多文件")
    @PostMapping("/uploadFiles")
    public AjaxResult uploadFile(@ApiParam(value = "文件", required = true) @RequestPart(value = "files") List<MultipartFile> files, @ApiParam(value = "空间ID", required = true) @RequestParam Long spaceId, @ApiParam(value = "目录ID", required = true) @RequestParam Long dirId) {
        try {
            mongoService.uploadFiles(files).stream().forEach(vo -> {
                process(vo, spaceId, dirId);
            });
            return success();
        } catch (Exception e) {
            return error(e.getMessage());
        }
    }

    /**
     * 文件上传之后的操作
     *
     * @param vo      上传对象
     * @param spaceId 空间ID
     * @param dirId   目录ID
     */
    private void process(DocumentVO vo, Long spaceId, Long dirId) {
        DocInfo docInfo = new DocInfo();
        docInfo.setSpaceId(spaceId);
        docInfo.setDirId(dirId);
        docInfo.setFileId(vo.getFileId());
        docInfo.setFileName(vo.getFileName());
        docInfo.setFileSize(vo.getFileSize());
        docInfo.setFileType(vo.getSuffix());
        docInfo.setCreateBy(SecurityUtils.getUsername());
        docInfoService.insertDocInfo(docInfo);
    }

    /**
     * 新建文件
     */
    @ApiOperation("新建文件")
    @GetMapping("/create")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "type", value = "文件类型 \"word\"：word文档 \"excel\"：excel文件 \"ppt\"：ppt文件  \"txt\"：文本文件", required = true),
            @ApiImplicitParam(name = "spaceId", value = "空间ID", required = true),
            @ApiImplicitParam(name = "dirId", value = "目录ID", required = true)
    })
    public AjaxResult create(Long spaceId, Long dirId, String type) {
        try {
            File file = null;
            String fileName = "";
            switch (type) {
                case "word":
                    file = ResourceUtils.getFile("classpath:file/new.docx");
                    fileName = "新建word文档.docx";
                    break;
                case "excel":
                    file = ResourceUtils.getFile("classpath:file/new.xlsx");
                    fileName = "新建Excel文档.xlsx";
                    break;
                case "ppt":
                    file = ResourceUtils.getFile("classpath:file/new.pptx");
                    fileName = "新建ppt文档.pptx";
                    break;
                case "txt":
                    file = ResourceUtils.getFile("classpath:file/new.txt");
                    fileName = "新建文本文档.txt";
                    break;
            }

            FileInputStream is = new FileInputStream(file);
            MultipartFile multipartFile = FileUtils.getMultipartFile(is, fileName);
            DocumentVO vo = mongoService.uploadFile(multipartFile);
            process(vo, spaceId, dirId);
            return success();
        } catch (Exception e) {
            log.error("新建文件出错啦", e.getMessage());
            return error("新建文件出错");
        }
    }

    /**
     * 文件移动
     */
    @ApiOperation("文件移动")
    @GetMapping("/move")
    @ApiImplicitParams({@ApiImplicitParam(name = "docId", value = "文件ID", required = true), @ApiImplicitParam(name = "spaceId", value = "空间ID"), @ApiImplicitParam(name = "dirId", value = "新目录ID", required = true)})
    public AjaxResult move(Long docId, Long dirId) {
        DocInfo info = docInfoService.selectDocInfoByDocId(docId);
        info.setDirId(dirId);

        info.setUpdateBy(SecurityUtils.getUsername());
        docInfoService.updateDocInfo(info);
        return success();
    }

    /**
     * 复制文件
     */
    @ApiOperation("文件复制")
    @GetMapping("/copy")
    @ApiImplicitParams({@ApiImplicitParam(name = "docId", value = "文件ID", required = true), @ApiImplicitParam(name = "spaceId", value = "空间ID"), @ApiImplicitParam(name = "dirId", value = "新目录ID", required = true)})
    public AjaxResult copy(Long docId, Long spaceId, Long dirId) {
        DocInfo info = docInfoService.selectDocInfoByDocId(docId);
        info.setDocId(null);
        info.setSpaceId(spaceId);
        info.setDirId(dirId);
        docInfoService.insertDocInfo(info);

        return success();
    }


    /**
     * 重命名
     */
    @ApiOperation("文件重命名")
    @GetMapping("/rename")
    @ApiImplicitParams({@ApiImplicitParam(name = "docId", value = "文件ID", required = true), @ApiImplicitParam(name = "name", value = "新文件名", required = true)})
    public AjaxResult rename(Long docId, String name) {
        DocInfo info = new DocInfo();
        info.setDocId(docId);
        info.setFileName(name);

        info.setUpdateBy(SecurityUtils.getUsername());

        return success(docInfoService.updateDocInfo(info));
    }

    /**
     * 查询文件基本信息表列表
     */
//    @ApiOperation("文件列表")
    //@PreAuthorize("@ss.hasPermi('biz:info:list')")
    @GetMapping("/list")
    public TableDataInfo list(DocInfo docInfo) {
        startPage();
        List<DocInfo> list = docInfoService.selectDocInfoList(docInfo);
        return getDataTable(list);
    }

    /**
     * 文件搜索
     */
    @ApiOperation("文件搜索")
    //@PreAuthorize("@ss.hasPermi('biz:info:list')")
    @GetMapping("/search")
    public TableDataInfo search(@ApiParam(value = "搜索关键字", required = true) @RequestParam String keyword) {
        Map<String, Object> temp = dirService.selectDirByUser(SecurityUtils.getUserId());
        List<DocDir> dirList = (List<DocDir>) temp.get("dir");
        List<DocSpace> spaceList = (List<DocSpace>) temp.get("space");
        Map<Long, DocDir> dirMap = new HashMap<>(16);
        Map<Long, DocSpace> spaceMap = new HashMap<>(16);
        List<Long> dirIds = new ArrayList<>();
        dirList.forEach(dir -> {
            dirIds.add(dir.getDirId());
            dirMap.put(dir.getDirId(), dir);
        });

        spaceList.forEach(space -> {
            spaceMap.put(space.getSpaceId(), space);
        });

        DocInfo docInfo = new DocInfo();
        docInfo.setFileName(keyword);
        if (dirIds.size() > 0) {
            Map<String, Object> map = new HashMap<>(4);
            map.put("dirIds", dirIds);
            docInfo.setParams(map);
        }
        startPage();
        List<DocInfo> list = docInfoService.selectDocInfoList(docInfo);
        List<DocInfoVO> result = new ArrayList<>();
        list.forEach(info -> {
            DocInfoVO vo = new DocInfoVO();
            BeanUtils.copyProperties(info, vo);

            vo.setDir(dirMap.get(vo.getDirId()));
            vo.setSpace(spaceMap.get(vo.getSpaceId()));

            result.add(vo);
        });
        return getDataTable(result);
    }

    /**
     * 查询文件基本信息表列表
     */
    @ApiOperation("根据目录ID查询文件列表")
    //@PreAuthorize("@ss.hasPermi('biz:info:list')")
    @GetMapping("/list/{dirId}")
    public TableDataInfo list4dir(@PathVariable("dirId") Long dirId) {
        startPage();
        DocInfo docInfo = new DocInfo();
        docInfo.setDirId(dirId);
        List<DocInfo> list = docInfoService.selectDocInfoList(docInfo);
        return getDataTable(list);
    }

    /**
     * 导出文件基本信息表列表
     */
//    @ApiOperation("导出文件基本信息表列表")
    //@PreAuthorize("@ss.hasPermi('biz:info:export')")
    @Log(title = "文件基本信息表", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, DocInfo docInfo) {
        List<DocInfo> list = docInfoService.selectDocInfoList(docInfo);
        ExcelUtil<DocInfo> util = new ExcelUtil<DocInfo>(DocInfo.class);
        util.exportExcel(response, list, "文件基本信息表数据");
    }

    /**
     * 获取文件基本信息表详细信息
     */
    @ApiOperation("文件详细信息")
    //@PreAuthorize("@ss.hasPermi('biz:info:query')")
    @GetMapping(value = "/{docId}")
    public AjaxResult getInfo(@PathVariable("docId") Long docId) {
        return success(docInfoService.selectDocInfoByDocId(docId));
    }

    /**
     * 新增文件基本信息表
     */
//    @ApiOperation("新增文件基本信息表")
    //@PreAuthorize("@ss.hasPermi('biz:info:add')")
    @Log(title = "文件基本信息表", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody DocInfo docInfo) {
        return toAjax(docInfoService.insertDocInfo(docInfo));
    }

    /**
     * 修改文件基本信息表
     */
//    @ApiOperation("修改文件基本信息表")
    //@PreAuthorize("@ss.hasPermi('biz:info:edit')")
    @Log(title = "文件基本信息表", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody DocInfo docInfo) {
        docInfo.setUpdateBy(SecurityUtils.getUsername());
        return toAjax(docInfoService.updateDocInfo(docInfo));
    }

    /**
     * 删除文件基本信息表
     */
    @ApiOperation("文件删除")
    //@PreAuthorize("@ss.hasPermi('biz:info:remove')")
    @Log(title = "文件基本信息表", businessType = BusinessType.DELETE)
    @DeleteMapping("/{docIds}")
    public AjaxResult remove(@PathVariable Long[] docIds) {
        for (Long docId : docIds) {
            DocInfo info = docInfoService.selectDocInfoByDocId(docId);
            //删除mongo记录
            mongoService.removeFile(info.getFileId());

            //删除ES记录
            try {
                String indexName = "docs_" + info.getSpaceId();
                esConfig.setIndexName(indexName);
                esDocInfoService.deleteById(docId);
            } catch (NoSuchIndexException e) {
                //不用管，表示没这个索引
            }
            //删除收藏记录
            favoriteService.delete("N", docId);
        }
        return toAjax(docInfoService.deleteDocInfoByDocIds(docIds));
    }


}
