package com.doc.biz.ws;

import com.alibaba.fastjson2.JSON;
import com.doc.chat.domain.ChatMsg;
import com.doc.chat.service.IChatMsgService;
import com.doc.chat.service.impl.ChatMsgServiceImpl;
import com.doc.common.constant.CacheConstants;
import com.doc.common.utils.spring.SpringUtils;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Component;

import javax.websocket.OnClose;
import javax.websocket.OnMessage;
import javax.websocket.OnOpen;
import javax.websocket.Session;
import javax.websocket.server.PathParam;
import javax.websocket.server.ServerEndpoint;
import java.io.IOException;
import java.util.concurrent.ConcurrentHashMap;

/**
 * websocket聊天
 *
 * @author wukai
 */
@ServerEndpoint(value = "/websocket/message/{uid}")
@Component
@Slf4j
public class WebSocketServer {
    private IChatMsgService msgService = SpringUtils.getBean(ChatMsgServiceImpl.class);
    private RedisTemplate redisTemplate = SpringUtils.getBean(StringRedisTemplate.class);
    /**
     * 保存 组id->组成员 的映射关系
     * 之所以使用ConcurrentHashMap因为这个是线程安全的
     */
    private static ConcurrentHashMap<Long, Session> currentMap = new ConcurrentHashMap<>();

    /**
     * 连接建立成功调用
     */
    @OnOpen
    public void onOpen(@PathParam("uid") Long uid, Session session) {
        //需要通知其他的客户端，将所有的用户的用户名发送给客户端
        //将当前连接的session存储到redis中
        currentMap.put(uid, session);
        log.info("用户(ID:{})建立连接!", uid);
        try {
            session.getBasicRemote().sendText("连接成功");
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 发布集群的消息
     *
     * @param msg
     */
    private void sendClusterMessage(String msg) {
        redisTemplate.convertAndSend(CacheConstants.CHANNEL, msg);
    }

    /**
     * 接收到客服端发送的数据消息时被调用
     */
    @OnMessage
    public void onMessage(@PathParam("uid") Long uid, String message) {
        try {
            //获取客户端发送来的数据  {"toId":2,"message":"你好"}
            ObjectMapper mapper = new ObjectMapper();
            ChatMsg msg = mapper.readValue(message, ChatMsg.class);
            msg.setFromId(uid);
            // 保存到数据库
            msgService.insertChatMsg(msg);
            //发布集群消息
            this.sendClusterMessage(message);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 连接关闭时调用
     */
    @OnClose
    public void onClose(@PathParam("uid") Long uid, Session session) {
        log.info("用户(ID:{})关闭连接!", uid);
        currentMap.remove(uid);
    }

    /**
     * 发布websocket消息
     *
     * @param msg
     * @return
     */
    public void sendClusterWebsocketMessage(ChatMsg msg) {
        if (msg != null) {
            //获取对方用户的连接会话
            try {
                Session session = currentMap.get(msg.getToId());
                if (session != null) {
                    session.getBasicRemote().sendText(JSON.toJSONString(msg));
                }
            } catch (Exception e) {
                log.info("会话异常！" + e.getMessage());
                e.printStackTrace();
            }
        }
    }
}
